library(Matrix)
library(tseries)
library(ada)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")


train_adaboost.batch <- function(iter=20, loss = "exponential") {
	map = replicate(20, 0)
	for(eventid in 1:20) {
		map[eventid] = train_adaboost(eventid, iter=iter, loss=loss)
	}
	return(map)
}

train_adaboost <- function(eventid, iter=20, loss = "exponential") {

	testidx = medtest.idx
	thistraining.posidx = ek10.training.posidx[[eventid]]
	thistraining.negidx = ek10.training.negidx[[eventid]]
	trainingidx = c(thistraining.posidx, thistraining.negidx)
	
	#training for each event
	training.labels = c(replicate(length(thistraining.posidx),1), replicate(length(thistraining.negidx),0))
	tsmat <- as.data.frame(feat[testidx,])
	trmat <- as.matrix(feat[trainingidx,])
	
	set.seed(0) 
	model <- ada(x = trmat, y = factor(training.labels), loss=loss, iter = iter)
	
	pred <- predict(model, newdata = tsmat, type = "prob")
	tsprediction <- pred[,2]
		
	testap = ap(cbind(labels[testidx,eventid], tsprediction))
	return(testap)
}


event.ids <- paste("E00",c(6:9),sep = "")
event.ids <- c(event.ids, paste("E0", c(10:15, 21:30), sep = ""))
event.names <- as.character(read.table("data/events")[,2])

idlist = read.table("data/medek10.idlist", header = FALSE, colClasses = "character")[,1]
splits = read.matrix("data/medek10.splits.new")
labels = read.matrix("data/medek10.labels.new")

load(file ="data/validation.posidx.obj")
load(file ="data/validation.negidx.obj")
medtest.idx = which(splits==1)

#in order to use liblinear efficiently, the dimension is reduced to 500 by PCA
feat = read.matrix("data/dcnnpca.feat")

ek10.training.posidx = list()
ek10.training.negidx = list()
for(eventid in 1:length(event.ids)) {
	ek10.training.posidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==1))
	ek10.training.negidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==0))
}


iter = c(30,50,70,90)
loss = c("exponential","logistic")

para.table = expand.grid(iter = iter, loss = loss)
para.map = replicate(nrow(para.table),0) 

for(i in 1:nrow(para.table)) {
	para.map[i] = mean(train_adaboost.batch(iter=para.table[i,1], loss = para.table[i,2]))
}
print(cbind(para.table, para.map))



#1   30 exponential 0.02055997
#2   50 exponential 0.02603704
#3   70 exponential 0.02820315
#4   90 exponential 0.02778087
#5   30    logistic 0.02301648
#6   50    logistic 0.02518188
#7   70    logistic 0.02645747
#8   90    logistic 0.02627975

map = train_adaboost.batch(iter=70, loss = "exponential")

print(mean(map))




