library(Matrix)
library(tseries)
library(LiblineaR)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")

batchtrain.all <- function(para.cost=1, para.epsilon=1) {
	map = replicate(20, 0)
	for(eventid in 1:20) {
		map[eventid] = batchtrain(eventid, para.cost=para.cost, para.epsilon=para.epsilon)
	}
	return(map)
}

batchtrain <- function(eventid, para.cost=1, para.epsilon=1) {
	testidx = medtest.idx
	trainingidx = c(ek10.training.posidx[[eventid]], ek10.training.negidx[[eventid]])
	trmat <- feat[trainingidx,]
	tsmat <- feat[testidx,]
	
	#training for each event
	training.labels = c(replicate(length(ek10.training.posidx[[eventid]]),1), replicate(length(ek10.training.negidx[[eventid]]),0))

	#liblinear
	trainset = scale(trmat,center=TRUE,scale=FALSE)
	#L2-regularized logistic regression is used since it has the best performance over other methods
	model <- LiblineaR(data=trainset, labels=training.labels, type=0, cost=para.cost, epsilon = para.epsilon, bias = TRUE, wi = NULL, verbose = FALSE)
	trprediction <- predict(model, trmat, decisionValues =TRUE)$decisionValues[,1]
	tsprediction <- predict(model, tsmat, decisionValues =TRUE)$decisionValues[,1]
	
	testap = ap(cbind(labels[testidx,eventid],tsprediction))
	
	return(testap)
}




lu = "med+batchtrain"
event.ids <- paste("E00",c(6:9),sep = "")
event.ids <- c(event.ids, paste("E0", c(10:15, 21:30), sep = ""))
event.names <- as.character(read.table("data/events")[,2])

idlist = read.table("data/medek10.idlist", header = FALSE, colClasses = "character")[,1]
splits = read.matrix("data/medek10.splits.new")
labels = read.matrix("data/medek10.labels.new")

medtest.idx = which(splits==1)

#in order to use liblinear efficiently, the dimension is reduced to 500 by PCA
feat = read.matrix("data/dcnnpca.feat")

ek10.training.posidx = list()
ek10.training.negidx = list()
for(eventid in 1:length(event.ids)) {
	ek10.training.posidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==1))
	ek10.training.negidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==0))
}


epsilon = 2^(-2:2)
para.table = expand.grid(epsilon = epsilon)
para.map = replicate(nrow(para.table),0) 

for(i in 1:nrow(para.table)) {
	para.map[i] = mean(batchtrain.all(para.cost=1, para.epsilon=para.table[i,1]))
}
print(cbind(para.table, para.map))




