library(Matrix)
library(tseries)
library(hash)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")


###this code prepare data for SPLD experiments######
event.ids <- paste("E00",c(6:9),sep = "")
event.ids <- c(event.ids, paste("E0", c(10:15, 21:30), sep = ""))
event.names <- as.character(read.table("data/events")[,2])

#labels[which(labels==-1059)]=0	#remove -1059
#dnn = read.matrix("data/featmat/med13ek100.dcnnfeatmap.mat")


ek100idlist = read.table("data/med13ek100.idlist", header = FALSE, colClasses = "character")[,1]
ek100splits = read.matrix("data/med13ek100.splits")
ek100labels = read.matrix("data/med13ek100.labels")

#find out the ek10 training example
{
	#1)ek10 training positive and negative
	ek10idlist = read.table("data/med13ek10.idlist", header = FALSE, colClasses = "character")[,1]
	ek10splits = read.matrix("data/med13ek10.splits")
	ek10labels = read.matrix("data/med13ek10.labels")
	
	
	ek10.training.pos = list()
	ek10.training.neg = list()
	for(i in 1:20) {
		ek10.training.pos[[i]] = ek10idlist[intersect(which(ek10splits==0), which(ek10labels[,i]==1))]
		ek10.training.neg[[i]] = ek10idlist[intersect(which(ek10splits==0), which(ek10labels[,i]==0))]
	}
	
	ek10.training.posidx = list()
	ek10.training.negidx = list()
	
	#2)find the ID in ek100 idlist
	h <- hash()
	.set(h, keys=ek100idlist, values=1:length(ek100idlist))
	for(i in 1:20) {
		ek10.training.posidx[[i]] = as.numeric(values(h, keys = ek10.training.pos[[i]]))
		ek10.training.negidx[[i]] = as.numeric(values(h, keys = ek10.training.neg[[i]]))
	}
	rm(h)
	
	
	
	#3)ek100 training positive and negative
	ek100.training.posidx = list()
	ek100.training.negidx = list()
	for(i in 1:20) {
		ek100.training.posidx[[i]] = intersect(which(ek100splits==0), which(ek100labels[,i]==1))
		ek100.training.negidx[[i]] = intersect(which(ek100splits==0), which(ek100labels[,i]==0))
	}
	
	
	#4)found out the validation set
	validation.posidx = list()
	validation.negidx = list()
	for(i in 1:20) {
		validation.posidx[[i]] = sample(setdiff(ek100.training.posidx[[i]], ek10.training.posidx[[i]]),10)
		validation.negidx[[i]] = ek10.training.negidx[[i]]
	}
	
	#5) found out the MEDTest set
	medtest.idx = list()
	for(i in 1:20) {
		medtest.idx[[i]] = which(ek100splits==1)
	}
	
}

trainingidx = c()
splits = matrix(-1, nrow = length(ek100idlist), ncol=1)
for(eventid in 1:length(event.ids)) {
	thisidx = union(ek10.training.posidx[[eventid]], ek10.training.negidx[[eventid]])
	#thisidx = ek10.training.posidx[[eventid]]
	trainingidx = union(trainingidx, thisidx)
}
splits[trainingidx,1] = 0	#training
splits[which(ek100splits[,1]==1),1] = 1	#test
write(splits, file = "medek10.splits.new", ncolumns=1)
#splits -1 not used; 0 for trianing; 1 for testing





###################################
#make a union label
labels = ek100labels
for(i in 1:20) {
	labels[ek10.training.posidx[[i]],i] = 1
	labels[ek10.training.negidx[[i]],i] = 0
}
write(t(labels), file = "medek10.labels.new", ncolumns=ncol(labels))







save(validation.posidx, file ="validation.posidx.obj")
save(validation.negidx, file ="validation.negidx.obj")
evl.idx = which(ek100splits==1)
save(evl.idx, file ="evl.idx.obj")




###############################################
###############################################
###############################################
dim=512
posidx = c()
negidx = c()
for(i in 1:20) {
	posidx = c(posidx, ek100.training.posidx[[i]])
	negidx = c(negidx, sample(ek100.training.negidx[[i]],100))
}

allidx = unique(union(posidx, negidx))
pca = prcomp(dnn[allidx,], center = TRUE)
#bug (not centerlized) replace with the  predict(pca, dnn)
dcnnpca <- dnn %*% pca$rotation[,1:dim]
write(t(dcnnpca), file = "dcnnpca.feat", ncol = ncol(dcnnpca))
