library(Matrix)
library(tseries)
library(randomForest)
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")


train_randforest.batch <- function(ntree=16, mtry = 16) {
	map = replicate(20, 0)
	for(eventid in 1:20) {
		map[eventid] = train_randforest(eventid, ntree=ntree, mtry=mtry)
	}
	return(map)
}


train_randforest <- function(eventid,  ntree=16, mtry = 16) {
	testidx = medtest.idx
	thistraining.posidx = ek10.training.posidx[[eventid]]
	thistraining.negidx = ek10.training.negidx[[eventid]]
	trainingidx = c(thistraining.posidx, thistraining.negidx)
	
	#training for each event
	training.labels = c(replicate(length(thistraining.posidx),1), replicate(length(thistraining.negidx),0))
	trmat <- as.matrix(feat[trainingidx,])
	tsmat <- as.matrix(feat[testidx,])
	
	set.seed(0) 
	#set the class weight to weight the labels in imbalanced data
	net <- randomForest(x = trmat, y = factor(training.labels), classwt = c(0.02,1), ntree=ntree, mtry=mtry)
	#net <- randomForest(x = trmat, y = factor(training.labels), ntree=ntree, mtry=mtry)
	
	pred<- predict(net, tsmat, type="vote")
	classes = attributes(pred)$dimnames[[2]]
	posclassidx = which(classes=="1")
	tsprediction = as.numeric(pred[,posclassidx])
	
	testap = ap(cbind(labels[testidx,eventid], tsprediction))
	#testap
	return(testap)
}


event.ids <- paste("E00",c(6:9),sep = "")
event.ids <- c(event.ids, paste("E0", c(10:15, 21:30), sep = ""))
event.names <- as.character(read.table("data/events")[,2])

idlist = read.table("data/medek10.idlist", header = FALSE, colClasses = "character")[,1]
splits = read.matrix("data/medek10.splits.new")
labels = read.matrix("data/medek10.labels.new")

load(file ="data/validation.posidx.obj")
load(file ="data/validation.negidx.obj")
medtest.idx = which(splits==1)

#in order to use liblinear efficiently, the dimension is reduced to 500 by PCA
feat = read.matrix("data/dcnnpca.feat")

ek10.training.posidx = list()
ek10.training.negidx = list()
for(eventid in 1:length(event.ids)) {
	ek10.training.posidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==1))
	ek10.training.negidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==0))
}

ntree = c(32,64,128,256,1024,2048)
mtry = c(16,32,64,128)

para.table = expand.grid(ntree = ntree, mtry = mtry)
para.map = replicate(nrow(para.table),0) 

for(i in 1:nrow(para.table)) {
	para.map[i] = mean(train_randforest.batch(ntree=para.table[i,1], mtry = para.table[i,2]))
}
cbind(para.table, para.map)


map = train_randforest.batch(ntree=2048, mtry = 128)
#you should be able to see similar results as following
#     ntree mtry    para.map
#1      4    2 0.003968336
#2      8    2 0.003994936
#3     16    2 0.007261738
#4     32    2 0.007297906
#5     64    2 0.006109840
#6      4    4 0.004012156
#7      8    4 0.004099607
#8     16    4 0.004395648
#9     32    4 0.004970752
#10    64    4 0.005703208
#11     4    8 0.003594932
#12     8    8 0.004186232
#13    16    8 0.005612334
#14    32    8 0.005298440
#15    64    8 0.005973500
#16     4   16 0.005048041
#17     8   16 0.005736245
#18    16   16 0.005956130
#19    32   16 0.007212504
#20    64   16 0.011143932
#21     4   32 0.004092694
#22     8   32 0.006420683
#23    16   32 0.009536187
#24    32   32 0.012029844
#25    64   32 0.013763485

#   ntree mtry    para.map
#1   256   16 0.01684318
#2   512   16 0.02106072
#3   768   16 0.02314128
#4   256   32 0.02561893
#5   512   32 0.02732710
#6   768   32 0.02837962
#1   768   64 0.02864327
#2  1024   64 0.02918246
#3  2048   64 0.03127312
#4   768  128 0.02916115
#5  1024  128 0.02972777
#6  2048  128 0.03123093





