library(Matrix)
library(tseries)
library(LiblineaR)
library(kernlab)
source("../selfpace_lib.r")
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")


selfpaced.diversity.train <- function(eventid, initialvals = NULL, diversity_ratio=0.01, clustermembership, pos.stepsize = 2, neg.stepsize = 10, para.epsilon=1) {
	testidx = medtest.idx
	devidx = c(ek10.training.posidx[[eventid]], ek10.training.negidx[[eventid]])
	validationidx = c(validation.posidx[[eventid]], ek10.training.negidx[[eventid]])
	
	trmat <- feat[devidx,]
	valmat <- feat[validationidx,]
	tsmat <- feat[testidx,]

	#
	temp = clustermembership[,1]
	dev_membership = unlist(lapply(devidx, function(x) {return(which(temp==x))}))
	dev_membership = clustermembership[dev_membership,2]
	
	dev_labels = c(replicate(length(ek10.training.posidx[[eventid]]),1), replicate(length(ek10.training.negidx[[eventid]]),0))
	val_labels = c(replicate(length(validation.posidx[[eventid]]),1), replicate(length(ek10.training.negidx[[eventid]]),0))
	evl_labels = labels[testidx,eventid]
	

	this.training.posidx = ek10.training.posidx[[eventid]]
	this.training.negidx = ek10.training.negidx[[eventid]]
	
	pos.stepsize = pos.stepsize
	neg.stepsize = neg.stepsize
	
	pos.lambda1_rank = pos.stepsize
	neg.lambda1_rank = neg.stepsize
	
	
	
	if(is.null(initialvals)) {
		this.training.posidx = sample(this.training.posidx, pos.lambda1_rank)
		this.training.negidx = sample(this.training.negidx, neg.lambda1_rank)
	} else {
		this.training.posidx = initialvals$pos
		this.training.negidx = initialvals$neg
	}
	
	initial.posidx = this.training.posidx
	initial.negidx = this.training.negidx
	
	
	iterusedpossamples = list()
	iterusednegsamples = list()
	
	valmaps = replicate(50,0)
	evlmaps = replicate(50,0)
	
	for(iter in 1:50) {
		print(paste("-----selected neg=", length(this.training.negidx), " pos=", length(this.training.posidx), sep = ""))
		
		thistraining.idx = c(this.training.posidx, this.training.negidx)
		thistraining.labels = c(replicate(length(this.training.posidx),1), replicate(length(this.training.negidx),0))
		
		iterusedpossamples[[iter]] = this.training.posidx
		iterusednegsamples[[iter]] = this.training.negidx
		
		
		#liblinear
		trainset = scale(feat[thistraining.idx,],center=TRUE,scale=FALSE)
		net <- LiblineaR(data=trainset, labels=thistraining.labels, type=0, cost=1, epsilon = para.epsilon, bias = TRUE, wi = NULL, verbose = FALSE)
		trprediction <- predict(net, trmat, proba =TRUE)$probabilities[,1]
		valprediction <- predict(net, valmat, proba =TRUE)$probabilities[,1]
		tsprediction <- predict(net, tsmat, proba =TRUE)$probabilities[,1]
		
		#self-paced learning with diversity
		selectedidx = selfpace2.rank(trprediction, dev_labels, dev_membership, pos.lambda1_rank, neg.lambda1_rank, diversity_ratio = diversity_ratio)
		
		
		#change the parameter accoding to the step size
		pos.lambda1_rank = pos.lambda1_rank + pos.stepsize
		neg.lambda1_rank = neg.lambda1_rank + neg.stepsize
		
		
		#evaluate the training and test error
		dev_loss = (trprediction-dev_labels)^2
		val_loss = (valprediction-val_labels)^2
		evl_loss = (tsprediction-evl_labels)^2
		evlrun = cbind(evl_labels,tsprediction)
		evlmap = ap(evlrun)
		
		valrun = cbind(val_labels,valprediction)
		valmap = ap(valrun)
		
		valmaps[iter] = valmap
		evlmaps[iter] = evlmap
		
		this.training.negidx = devidx[unique(selectedidx$neg)]
		this.training.posidx = devidx[unique(selectedidx$pos)]
		print(paste("iter ", iter, ":" , " devloss=",  mean(dev_loss), " evlloss=", mean(evl_loss), " valmap=", valmap, " evlmap=", evlmap, sep = ""))
	}
	
	#3)find the run with the best valudation map
	bestiter = which(valmaps==max(valmaps[1:length(valmaps)]))
	
	initialvals = list("pos" = initial.posidx, "neg" = initial.negidx)
	results <- list("bestiter" = bestiter, "devevl" = cbind(valmaps, evlmaps), "initialvals" = initialvals, "iterusedpossamples" = iterusedpossamples, "iterusednegsamples"=iterusednegsamples)
	
	return(results)
}



lu = "med+spld"
event.ids <- paste("E00",c(6:9),sep = "")
event.ids <- c(event.ids, paste("E0", c(10:15, 21:30), sep = ""))
event.names <- as.character(read.table("data/events")[,2])

idlist = read.table("data/medek10.idlist", header = FALSE, colClasses = "character")[,1]
splits = read.matrix("data/medek10.splits.new")
labels = read.matrix("data/medek10.labels.new")

load(file ="data/validation.posidx.obj")
load(file ="data/validation.negidx.obj")
medtest.idx = which(splits==1)

#in order to use liblinear efficiently, the dimension is reduced to 500 by PCA
feat = read.matrix("data/dcnnpca.feat")

ek10.training.posidx = list()
ek10.training.negidx = list()
for(eventid in 1:length(event.ids)) {
	ek10.training.posidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==1))
	ek10.training.negidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==0))
}


#######################################################################
#######################################################################
### if you cannot get the map in our paper                          ###
### probably due to the randomness in starting values/clustering    ###
### please run the code below to load our starting values           ###
### which can generate the same output                              ###
#######################################################################
#######################################################################
#1) load the starting values
load("data/spld.initialvalues.obj")

#2) load the membership/stepsize/bestrunid
mymembership = read.matrix("data/clustermembership.matrix")
spld.stepsizes = t(cbind(replicate(10,c(0.01,1,100)),replicate(5,c(0.01,2,150)),replicate(10,c(0.1,1,100)),replicate(5,c(0.1,2,150))))	#tunning the step size in the method
spld.bestrunid = read.table("data/spld+bestrunids.txt")[,1]

#3) rerun the best results
spld.allresults <- list()
for(eventid in 1:20) {
	print(eventid)
	bestrunid = spld.bestrunid[eventid]
	spld.allresults[[eventid]] = selfpaced.diversity.train(eventid, initialvals = spld.initialvalues[[eventid]][[bestrunid]], diversity_ratio = spld.stepsizes[bestrunid,1], clustermembership = mymembership, pos.stepsize = spld.stepsizes[bestrunid,2], neg.stepsize = spld.stepsizes[bestrunid,3])
}

#3) eveluate
spld.bestmap <- replicate(20,0)
for(eventid in 1:20) {
	spld.bestmap[eventid] = spld.allresults[[eventid]]$devevl[spld.allresults[[eventid]]$bestiter[1],2] 	#choose the evlaution map of the best run
}
print(cbind(event.names, spld.bestmap))
print(mean(spld.bestmap))

