library(Matrix)
library(tseries)
library(LiblineaR)
library(kernlab)
source("../selfpace_lib.r")
source("http://lu-library.googlecode.com/svn/trunk/evaluation.r")




lu = "med+spld"
event.ids <- paste("E00",c(6:9),sep = "")
event.ids <- c(event.ids, paste("E0", c(10:15, 21:30), sep = ""))
event.names <- as.character(read.table("data/events")[,2])

idlist = read.table("data/medek10.idlist", header = FALSE, colClasses = "character")[,1]
splits = read.matrix("data/medek10.splits.new")
labels = read.matrix("data/medek10.labels.new")

load(file ="data/validation.posidx.obj")
load(file ="data/validation.negidx.obj")
medtest.idx = which(splits==1)

#in order to use liblinear efficiently, the dimension is reduced to 500 by PCA
feat = read.matrix("data/dcnnpca.feat")

ek10.training.posidx = list()
ek10.training.negidx = list()
for(eventid in 1:length(event.ids)) {
	ek10.training.posidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==1))
	ek10.training.negidx[[eventid]] = intersect(which(splits==0), which(labels[,eventid] ==0))
}


#######################################################################
#######################################################################
### another way to reproduce our final model                        ###
### only train the final model only using the final sample 		    ###
#######################################################################
#######################################################################
aps = replicate(length(event.ids),0)
#the samples used in the final models
sampleset = read.matrix("med.spld.besttrainingset.matrix")
testidx = which(splits==1)
tsmat <- feat[testidx,]
for(eventid in 1:length(event.ids)) {
	thistraining.idx = which(sampleset[,eventid] != -1059)
	thistraining.labels = labels[thistraining.idx,eventid]
	
	trainset = scale(feat[thistraining.idx,],center=TRUE,scale=FALSE)
	model <- LiblineaR(data=trainset, labels=thistraining.labels, type=0, cost=1, epsilon = 1, bias = TRUE, wi = NULL, verbose = FALSE)
	
	tsprediction <- predict(model, tsmat, proba =TRUE)$probabilities
	
	#For liblinear. It treats the first seen label as the column one.
	#Here we are interested in finding the column for +1
	poslabelcolumn = which(colnames(tsprediction)=="1")
	tsprediction = tsprediction[,poslabelcolumn]
	
	aps[eventid] = ap(cbind(labels[testidx,eventid], tsprediction))
}
print(cbind(1:length(aps), aps))
print(mean(aps))






